/** @file Animation.h
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CAnimation
 *
 *
 * \brief Animace
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef ANIMATION_H_
#define ANIMATION_H_

class CAnimation {
public:
	/**
	 * Konstruktor
	 */
	CAnimation();

	/**
	 * Destruktor
	 */
	virtual ~CAnimation();

	/**
	 * Vrátí číslo aktuálního snímku
	 * @return
	 */
	int getCurrentFrame() const {
		return m_currentFrame;
	}

	/**
	 * Vrátí číslo posledního snímku
	 * @return
	 */
	int getEnd() const {
		return m_end;
	}

	/**
	 * Vrátí zdali je animace přehrávána
	 * @return
	 */
	bool isPlay() const {
		return m_play;
	}

	/**
	 * Vrátí číslo prvního snímek
	 * @return
	 */
	int getStart() const {
		return m_start;
	}

	/**
	 * Vrátí velikost kroku
	 * @return
	 */
	int getStep() const {
		return m_step;
	}

	/**
	 * Nastaví poslední snímek
	 * @param m_end - koncový snímek
	 */
	void setEnd(int m_end) {
		this->m_end = m_end;
	}

	/**
	 * Nastaví první snímek
	 * @param m_start - počáteční snímek
	 */
	void setStart(int m_start) {
		this->m_start = m_start;
	}

	/**
	 * Nastaví velikost kroku animace
	 * @param m_step - krok animace
	 */
	void setStep(int m_step) {
		this->m_step = m_step;
	}

	/**
	 * Spustí animaci
	 */
	void play() {
		m_play = true;
	}

	/**
	 * Zastaví animaci
	 */
	void stop() {
		m_play = false;
	}

	/**
	 * Přejdi na další snímek
	 */
	void nextFrame() {
		this->m_currentFrame += m_step;
	}

	/**
	 * Aktualizace animace
	 */
	void update();

	/**
	 * Reset animace - přetočení na počáteční číslo snímku
	 */
	void reset() {
		m_currentFrame = m_start;
	}

	/**
	 * Vratí, zdali je konec animace
	 */
	bool isEnd() {
		return m_end <= m_currentFrame ? true : false;
	}

	/**
	 * Komplexní nastavení animace
	 * @param m_play - přehát animaci?
	 * @param m_start - počáteční snímek
	 * @param m_end - koncový snímek
	 * @param m_step - krok animace
	 */
	void setup(bool m_play, int m_start, int m_end, int m_step);

protected:

	/**
	 * Je animace přehrávána?
	 */
	bool m_play;

	/**
	 * Krok animace - tj. počet snímků
	 */
	int m_step;

	/**
	 * Číslo počátečního snímku
	 */

	int m_start;

	/**
	 * Číslo posledního snímku
	 */
	int m_end;

	/**
	 * Číslo aktuálního snímku
	 */

	int m_currentFrame;

};

#endif /* ANIMATION_H_ */
