/** @file Camera.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CCamera
 *
 *
 * \brief Kamera
 *
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CCAMERA_
#define _CCAMERA_

#include "structures.h"
#include "Singleton.h"

class CCamera: public CSingleton<CCamera> {

public:

	/**
	 * Nastaví pozici
	 * @param position
	 */
	void setPosition(Vertex3f position) {
		m_position = position;
	}

	/**
	 * Nastaví rotaci
	 * @param rotation
	 */
	void setRotation(Vertex3f rotation) {
		m_rotation = rotation;
	}

	/**
	 * Nastaví cílový bod
	 * @param position
	 */
	void setTarget(Vertex3f position) {
		m_target = position;
	}

	/**
	 * Nastaví pozici
	 * @param x
	 * @param y
	 * @param z
	 */
	void setPosition(float x, float y, float z) {
		m_position . x = x;
		m_position . y = y;
		m_position . z = z;
	}

	/**
	 * Nastaví rotaci
	 * @param x
	 * @param y
	 * @param z
	 */
	void setRotation(float x, float y, float z) {
		m_rotation . x = x;
		m_rotation . y = y;
		m_rotation . z = z;
	}

	/**
	 * Nastaví cílový bod
	 * @param x
	 * @param y
	 * @param z
	 */
	void setTarget(float x, float y, float z) {
		m_target . x = x;
		m_target . y = y;
		m_target . z = z;
	}

	/**
	 * Nastaví pozici x kamery
	 * @param x
	 */
	void setPositionX(float x) {
		m_position . x = x;
	}

	/**
	 * Nastaví pozici z kamery
	 * @param y
	 */
	void setPositionY(float y) {
		m_position . y = y;
	}

	/**
	 * Nastaví pozici z kamery
	 * @param z
	 */
	void setPositionZ(float z) {
		m_position . z = z;
	}

	/**
	 * Nastaví pozici x cílového bodu
	 * @param x
	 */
	void setTargetX(float x) {
		m_target . x = x;
	}

	/**
	 * Nastaví pozici y cílového bodu
	 * @param y
	 */
	void setTargetY(float y) {
		m_target . y = y;
	}

	/**
	 * Nastaví pozici y cílového bodu
	 * @param y
	 */
	void setTargetZ(float z) {
		m_target . z = z;
	}

	/**
	 * Posune kameru po ose x o velikost step
	 * @param x
	 */
	void movePositionX(float step) {
		m_position . x += step;
	}

	/**
	 * Posune kameru po ose y o velikost step
	 * @param y
	 */
	void movePositionY(float step) {
		m_position . y += step;
	}

	/**
	 * Posune kameru po ose y o velikost step
	 * @param y
	 */
	void movePositionZ(float step) {
		m_position . z += step;
	}

	/**
	 * Nastaví úhel kamery
	 * @param angle
	 */
	void setAngle(GLfloat angle) {
		m_angle = angle;
	}

	/**
	 * Vrátí úhel kamery
	 * @return
	 */
	GLfloat getAngle() {
		return m_angle;
	}

	/**
	 * Nastaví pozici kamery
	 */
	void look();

private:

	/**
	 * Pozice kamery
	 */
	Vertex3f m_position;

	/**
	 * Rotace kamery
	 */
	Vertex3f m_rotation;

	/**
	 * Cíl kamery
	 */
	Vertex3f m_target;

	/**
	 * Úhel kamery
	 */
	GLfloat m_angle;

};

#endif

