/** @file Colorable.h
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CColorable
 *
 *
 * \brief Obarvení objektu
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef CCOLORABLE_H_
#define CCOLORABLE_H_

#include "main.h"
#include "structures.h"

class CColorable {
public:

	CColorable();
	virtual ~CColorable();

	/**
	 * Nastaví barvu
	 * @param color
	 */
	void setColor(Color4f color) {
		m_color = color;
	}

	/**
	 * Nastaví barvu
	 * @param red
	 * @param green
	 * @param blue
	 * @param alpha
	 */
	void setColor(float red, float green, float blue, float alpha);

	/**
	 * Nastaví hodnotu červené složky barvy
	 * @param red
	 */

	void setRed(float red) {
		m_color.r = red / 255.0f;
	}

	/**
	 * Nastaví hodnotu zelené složky barvy
	 * @param red
	 */
	void setGreen(float green) {
		m_color.g = green / 255.0f;
	}

	/**
	 * Nastaví hodnotu modré složky barvy
	 * @param red
	 */
	void setBlue(float blue) {
		m_color.b = blue / 255.0f;
	}

	/**
	 * Nastaví míru průhlednosti
	 * @param red
	 */
	void setAlpha(float alpha) {
		m_color.alpha = alpha / 255.0;
	}

	/**
	 * Vrátí barvu
	 * @param red
	 */
	Color4f getColor(void) const {
		return m_color;
	}

	/**
	 * Vrátí hodnotu červené složky barvy v rozmezí od <0.0,1.0>
	 * @param red
	 */
	float getRed(void) const {
		return m_color.r;
	}

	/**
	 * Vrátí hodnotu zelené složky barvy v rozmezí od <0.0,1.0>
	 * @param red
	 */
	float getGreen(void) const {
		return m_color.g;
	}

	/**
	 * Vrátí hodnotu modré složky barvy v rozmezí od <0.0,1.0>
	 * @param red
	 */
	float getBlue(void) const {
		return m_color.b;
	}

	/**
	 * Vrátí hodnotu míry průhlednosti v rozmezí od <0.0,1.0>
	 * @param red
	 */
	float getAlpha(void) const {
		return m_color.alpha;
	}

protected:
	Color4f m_color;

};

#endif /* CCOLORABLE_H_ */
