#include "Cube.h"

#include "TextureManager.h"

CGOCube::CGOCube(void){
	m_vertexes = new GLfloat[ 72 ];

	GLfloat vertexes[] = {
			1,1,1,  0,1,1,  0,0,1,  1,0,1,        	// v0-v1-v2-v3
			1,1,1,  1,0,1,  1,0,0,  1,1,0,        	// v0-v3-v4-v5
			1,1,1,  1,1,0,  0,1,0,  0,1,1,        	// v0-v5-v6-v1
			0,1,1,  0,1,0,  0,0,0,  0,0,1,    	// v1-v6-v7-v2
			0,0,0,  1,0,0,  1,0,1,  0,0,1,    	// v7-v4-v3-v2
			1,0,0,  0,0,0,  0,1,0,  1,1,0   	// v4-v7-v6-v5
	};

	GLfloat texCoord[] = { 0, 0,   1,0,  1,1,   0,1,
						  0, 0,   1,0,  1,1,   0,1,
						  0, 0,   1,0,  1,1,   0,1,
						  0, 0,   1,0,  1,1,   0,1,
						  0, 0,   1,0,  1,1,   0,1,
						  0, 0,   1,0,  1,1,   0,1
	};

	for( int i = 0; i < 72; i++ ){
		m_vertexes [ i ] = vertexes[ i ];
	}

	this->setRotation(0,0,0);
	this->setPosition(0,0,0);
	this->setDimension(1, 1, 1);

	this->setVertices( texCoord, 48 );

}

CGOCube::~CGOCube(void){
	if( m_vertexes != NULL ){
		delete [] m_vertexes;
	}
}

/** Vykresleni kostky
 */
void CGOCube::render(){

	glPushMatrix();	// ulozime matici na zasobnik

	// posuneme objekt na pozici
	glTranslatef( getPositionX(), getPositionY(), getPositionZ());

	// nastavime rotaci
	if( isRotated() ){
		glRotatef( m_rotation.rotX, 1.0, 0.0, 0.0 );
		glRotatef( m_rotation.rotY, 0.0, 1.0, 0.0 );
		glRotatef( m_rotation.rotZ, 0.0, 0.0, 1.0 );
	}

	glScalef( getWidth() , getHeight(), getLength() );

	// zvetsime objekt na pozadovanou velikost

	glEnableClientState(GL_VERTEX_ARRAY);
	glColor4f( getRed(), getGreen(), getBlue(), getAlpha());

	if( isTextured() && isSetVertices() ){

		glEnable(GL_TEXTURE_2D);

		glBindTexture( GL_TEXTURE_2D, CTextureManager::getInstance()->get(getTextureId())->getID() );
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, m_texCoord);
	}

	glVertexPointer(3, GL_FLOAT, 0, m_vertexes);
	glDrawArrays(GL_QUADS, 0, 24);

	if( this->isTextured() && isSetVertices() ){
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_2D);
	}

	glDisableClientState(GL_VERTEX_ARRAY);



	glPopMatrix();	// vyhodime matici ze zasobniku

}

