/** @file GUIButton.h
*
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGUIButton
 *
 *
 * \brief GUI tlačítko
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CGUIBUTTON_
#define _CGUIBUTTON_

#include "GUIElement.h"

template<class T>
class CGUIButton : public CGUIElement {

public:
	CGUIButton();
	CGUIButton(T* _instance);

	/**
	 * Vykreslí tlačítko
	 */
	void render();

	/**
	 * Aktualizace tlačítka
	 */
	void update();

	/**
	 *
	 * @param button
	 * @param state
	 * @param x
	 * @param y
	 */
	void mouseButtons(int button, int state, int x, int y);

	// callbacks

	/**
	 * Nastaví callback na uvolnění tlačítka myši
	 * @param _func
	 */

	void setMouseUpCallback(void(T::*_func)(void)) {
		m_mouseUpCallback = _func;
	}

	void setMouseDownCallback(void(T::*_func)(void)) {
		m_mouseDownCallback = _func;
	}

	void setCallbackClass(T* _instance) {
		m_instance = _instance;
	}

	void setCallback(T* _instance, void(T::*_funcMouseDown)(void),
			void(T::*_funcMouseUp)(void)) {
		m_instance = _instance;
		m_mouseDownCallback = _funcMouseDown;
		m_mouseUpCallback = _funcMouseUp;
	}

protected:

	T* m_instance;

	void (T::*m_mouseUpCallback)(void);
	void (T::*m_mouseDownCallback)(void);

};

template<class T>
CGUIButton<T>::CGUIButton() {
	m_instance = NULL;

	m_mouseUpCallback = NULL;
	m_mouseDownCallback = NULL;

	this->setBackgroundColor(0,0,0,0);
	this->setBackgroundHoverColor(0,0,0,0);
	this->setTextColor(255, 255, 255, 255);
	this->setTextHoverColor(100, 100, 100, 255);
}

template<class T>
CGUIButton<T>::CGUIButton(T* _instance) {
	m_instance = _instance;
	m_mouseUpCallback = NULL;
	m_mouseDownCallback = NULL;

	this->setBackgroundColor(0,0,0,0);
	this->setBackgroundHoverColor(0,0,0,0);
	this->setTextColor(255, 255, 255, 255);
	this->setTextHoverColor(200, 200, 200, 255);

}

template<class T>
void CGUIButton<T>::render() {

	glPushMatrix();

	glTranslatef(getPositionX(), getPositionY(), 0);

	Color4f backgroundColor = this->getCurrentBackgroundColor();

	glColor4f(backgroundColor.r, backgroundColor.g, backgroundColor.b,
			backgroundColor.alpha);

	glBegin(GL_QUADS);
	glVertex2f(0, 0);
	glVertex2f(getWidth(), 0);
	glVertex2f(getWidth(), getHeight());
	glVertex2f(0, getHeight());
	glEnd();

	glPopMatrix();

	CFontPrinter::getInstance()->print(getFontId(), getPositionX(),
			getPositionY(), this->getCurrentTextColor(), this->getString());

}

template<class T>
void CGUIButton<T>::mouseButtons(int button, int state, int x, int y) {

	if (this->isHover()) {
		if (m_instance != NULL) {
			if (state == GLUT_UP && this->m_mouseUpCallback != NULL) {
				(this->m_instance->*m_mouseUpCallback)();

			} else if (state == GLUT_DOWN && this->m_mouseDownCallback != NULL) {
				(this->m_instance->*m_mouseDownCallback)();
			}
		}
	}

}

template<class T>
void CGUIButton<T>::update() {

}

#endif
