/** @file GUIConfirm.h
*
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGUIConfirm
 *
 *
 * \brief GUI otázka
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef GUICONFIRM_H_
#define GUICONFIRM_H_

#include "GUIInput.h"
#include "GUIButton.h"
#include "GUIFrame.h"

template<class T>
class CGUIConfirm: public CGUIFrame {
public:
	CGUIConfirm();
	virtual ~CGUIConfirm();

	void setConfirmMessage(string message);
	void setCallbackInstance(T* _instance) {
		m_instance = _instance;
	}
	;
	void setResultCallback(void(T::*_func)(bool)) {
		m_funcResult = _func;
	}
	;

	void setCallback(T* _instance, void(T::*_func)(bool)) {
		setCallbackInstance(_instance);
		setResultCallback(_func);
	}

	virtual void initChilds();
	virtual void centered();

protected:

	void onConfirm();
	void onCancel();

	T* m_instance;
	void (T::*m_funcResult)(bool);

};

template<class T>
CGUIConfirm<T>::CGUIConfirm() {
	// TODO Auto-generated constructor stub
	m_instance = NULL;
	m_funcResult = NULL;
}

template<class T>
CGUIConfirm<T>::~CGUIConfirm() {
	// TODO Auto-generated destructor stub
}

template<class T>
void CGUIConfirm<T>::initChilds() {
	CGUILabel* text = new CGUILabel();
	text->setColor(200, 200, 200, 255);

	CGUIButton<CGUIConfirm> *buttonOK = new CGUIButton<CGUIConfirm> (this);
	buttonOK->setMouseDownCallback(&CGUIConfirm::onConfirm);
	buttonOK->setString("OK");
	buttonOK->setDimension(25, 14);

	CGUIButton<CGUIConfirm> *buttonCancel = new CGUIButton<CGUIConfirm> (this);
	buttonCancel->setMouseDownCallback(&CGUIConfirm::onCancel);
	buttonCancel->setString("CANCEL");
	buttonCancel->setDimension(50, 14);

	this->addChild(text);
	this->addChild(buttonOK);
	this->addChild(buttonCancel);
}

template<class T>
void CGUIConfirm<T>::onCancel() {

	bool m_result = false;

	if (m_instance != NULL && m_funcResult != NULL) {
		(m_instance->*m_funcResult)(m_result);
	}

}

template<class T>
void CGUIConfirm<T>::onConfirm() {

	bool m_result = true;

	if (m_instance != NULL && m_funcResult != NULL) {
		(m_instance->*m_funcResult)(m_result);
	}

}

template<class T>
void CGUIConfirm<T>::setConfirmMessage(string message) {
	CGUIElement* element = NULL;
	if ((element = getChild(0)) != NULL) {
		element->setString(message);
	}
}

template<class T>
void CGUIConfirm<T>::centered() {

	GLint viewport[4]; 						// informace o viewportu
	glGetIntegerv(GL_VIEWPORT, viewport); 	// ziskame informace o VP

	GLfloat positionX = (viewport[2] / 2) - (this->getWidth() / 2);
	GLfloat positionY = (viewport[3] / 2) - (this->getHeight() / 2);

	setPosition(positionX, positionY);

	CGUIElement* curr = NULL;

	for( int i = 1; i < size(); i++ ){
		if( ( curr = getChild(i) ) != NULL ){
			GLfloat px = positionX + (getWidth()/2) - curr->getWidth() / 2 ;
			curr->setPositionX( px );
		}
	}
/*
	CGUIElement* buttonOK = getChild(1);
	CGUIElement* buttonCancel = getChild(2);

	if(buttonOK != NULL){
		GLfloat px = positionX + (getWidth()/2) - buttonOK->getWidth() / 2 ;
		buttonOK->setPositionX( px );
	}

	if(buttonCancel != NULL){
		GLfloat px = positionX + (getWidth()/2) - buttonCancel->getWidth() / 2;
		buttonCancel->setPositionX( px );
	}*/

}

#endif /* GUICONFIRM_H_ */
