/** @file GUIElement.h
*
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGUIElement
 *
 *
 * \brief Základní GUI Element
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CGUIELEMENT_
#define _CGUIELEMENT_

#include "Stringable.h"
#include "GameObject2D.h"
#include "FontManager.h"
#include "FontPrinter.h"
#include "GUIHoverColorable.h"

class CGUIElement: public CStringable, public CGameObject2D, public CGUIHoverColorable {

public:

	CGUIElement() {
		m_isHover = false;
		m_fontId = -1;
	}

	/**
	 * Nastaví řetězec návěští
	 * @param label
	 */
	void setLabel(string str) {
		m_label = str;
	}

	/**
	 * Nastaví řetězec návěští
	 * @param str
	 */
	void setLabel(const char *str) {
		m_label = string(str);
	}

	/**
	 * Vratí řetězec návěští
	 * @return
	 */
	string getLabel() {
		return m_label;
	}

	/**
	 * Nastaví identifikátoru písma
	 * @param fontId
	 */
	void setFontId(int fontId) {
		m_fontId = fontId;
	}

	/**
	 * Vrátí identifikátor písma
	 * @return
	 */
	int getFontId() {
		return m_fontId;
	}

	bool isHover(){ return m_isHover; }

	// eventy

	/**
	 *
	 * @param x
	 * @param y
	 */
	void mouseMove(int x, int y) {
		if( this->isVisible() ){
			if (x >= getPositionX() && x <= getPositionX() + getWidth() && y
					>= getPositionY() && y <= getPositionY() + getHeight()) {
				m_isHover = true;
			}else{
				m_isHover = false;
			}
		}
	}

	Color4f getCurrentTextColor(){
		if(isHover()){
			return this->m_textHover.getColor();
		}

		return this->m_text.getColor();
	}

	Color4f getCurrentBackgroundColor(){
		if(isHover()){
			return this->m_backgroundHover.getColor();
		}

		return this->m_background.getColor();
	}

	/**
		 * Vrátí vnitřní odsazení zleva
		 * @return
		 */
		GLfloat getPaddingLeft() const { return m_paddingLeft; };

		/**
		 * Vrátí vnitřní odzasení odshora
		 * @return
		 */
		GLfloat getPaddingTop() const{ return m_paddingTop; };

		/**
		 * Vrátí výšku řádku
		 * @return
		 */
		GLfloat getLineHeight() const{ return m_lineHeight; };

		/**
		 * Nastaví odsazení zleva
		 * @param value
		 */
		void 	setPaddingLeft( GLfloat value ){ m_paddingLeft = value; };

		/**
		 * Nastaví odsazení odshora
		 * @param vlaue
		 */
		void	setPaddingTop( GLfloat value ){ m_paddingTop = value; };

		/**
		 * Nastaví výšku řádky
		 * @param height
		 */
		void	setLineHeight( GLfloat height ){ m_lineHeight = height; };

protected:

	/**
	 * Návěští elementu
	 */
	string m_label;

	/**
	 * Identifikátor písma
	 */
	int m_fontId;

	/**
	 * Příznak, zdali je kurzor nad elementem
	 */
	bool m_isHover;


	GLfloat m_paddingLeft;
	GLfloat m_paddingTop;
	GLfloat	m_lineHeight;

};

#endif
