/** @file GUIHoverColorable.h
*
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGUIHoverColorable
 *
 *
 * \brief Obarvení objektů při událostech hover(najetí myši na objekt)
 *
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef GUIHOVERCOLORABLE_H_
#define GUIHOVERCOLORABLE_H_

#include "Colorable.h"

class CGUIHoverColorable {

public:

	CGUIHoverColorable();
	~CGUIHoverColorable();

	/**
	 * Nastavi barvu pozadi
	 * @param color
	 */
	void setBackgroundColor(Color4f color) {
		m_background.setColor(color);
	}

	/**
	 * Nastavi barvu pozadi
	 * @param red
	 * @param green
	 * @param blue
	 * @param alpha
	 */
	void setBackgroundColor(float red, float green, float blue, float alpha) {
		m_background.setColor(red, green, blue, alpha);
	}

	/**
	 * Nastavi barvu pozadi - najeti mysi nad objekt
	 * @param color
	 */
	void setBackgroundHoverColor(Color4f color) {
		m_backgroundHover.setColor(color);
	}

	/**
	 * Nastavi barvu pozadi - najeti mysi nad objekt
	 * @param red
	 * @param green
	 * @param blue
	 * @param alpha
	 */
	void setBackgroundHoverColor(float red, float green, float blue,
			float alpha) {
		m_backgroundHover.setColor(red, green, blue, alpha);
	}

	/**
	 * Nastavi barvu textu
	 * @param color
	 */
	void setTextColor(Color4f color) {
		m_text.setColor(color);
	}

	/**
	 * Nastavi barvu textu
	 * @param red
	 * @param green
	 * @param blue
	 * @param alpha
	 */
	void setTextColor(float red, float green, float blue, float alpha) {
		m_text.setColor(red, green, blue, alpha);
	}

	/**
	 * Nastavi barvu textu - najeti mysi nad objekt
	 * @param color
	 */
	void setTextHoverColor(Color4f color) {
		m_textHover.setColor(color);
	}

	/**
	 * Nastavi barvu textu - najeti mysi nad objekt
	 * @param red
	 * @param green
	 * @param blue
	 * @param alpha
	 */
	void setTextHoverColor(float red, float green, float blue, float alpha) {
		m_textHover.setColor(red, green, blue, alpha);
	}

	Color4f getTextColor() const { return m_text.getColor(); };
	Color4f getTextHoverColor() const { return m_textHover.getColor(); };
	Color4f getBackgroundColor() const { return m_background.getColor(); };
	Color4f getBackgroundHoverColor() const { return m_backgroundHover.getColor(); };

protected:
	CColorable m_background;
	CColorable m_backgroundHover;
	CColorable m_text;
	CColorable m_textHover;

};

#endif /* GUIHOVERCOLORABLE_H_ */
