/** @file GUIListOption.h
*
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGUIListOption
 *
 *
 * \brief GUI prvek seznamu
 *
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CGUILISTOPTION_
#define _CGUILISTOPTION_

#include "GUIElement.h"
#include "GUIList.h"

template<class Class, typename T>
class CGUIListOption: public CGUIElement {

public:

	CGUIListOption();

	void render();

	void setValue(int value) {
		m_value = value;
	}

	int getValue() const {
		return m_value;
	}

	void update() {
	}

	void active() {
		m_active = true;
	}

	void nonactive() {
		m_active = false;
	}

	bool isActive() {
		return m_active;
	}

	void setID(int id) {
		m_ID = id;
	}

	void mouseButtons(int button, int state, int x, int y);

	void setParent(Class* parent) {
		m_parent = parent;
	}

	void setParentOnActivated(void(Class::*func)(int)) {
		m_func = func;
	}

protected:

	T m_value;
	bool m_active;
	int m_ID;

	Color4f m_colorActive;
	Color4f m_colorTextActive;

	Class* m_parent;
	void (Class::*m_func)(int);

};

template<class C, typename T>
CGUIListOption<C, T>::CGUIListOption() {
	this->setDimension(96, 20);
	this->setBackgroundColor(50, 50, 50, 128);
	this->setBackgroundHoverColor(0, 0, 0, 128);
	this->setTextHoverColor(255, 255, 255, 255);

	m_colorActive.r = 1;
	m_colorActive.b = 1;
	m_colorActive.g = 1;
	m_colorActive.alpha = 0.5;

	this->m_active = false;

}

template<class C, typename T>
void CGUIListOption<C, T>::render() {

	CFontPrinter* fontPrinter = CFontPrinter::getInstance();
	fontPrinter->activeFont(this->getFontId());

	Color4f color;

	glPushMatrix();

	glTranslatef(getPositionX(), getPositionY(), 0);

	Color4f backgroundColor = this->getCurrentBackgroundColor();

	if (this->isActive()) {
		backgroundColor = m_colorActive;
	}

	glColor4f(backgroundColor.r, backgroundColor.g, backgroundColor.b,
			backgroundColor.alpha);

	glBegin(GL_QUADS);
	glVertex2f(0, 0);
	glVertex2f(getWidth(), 0);
	glVertex2f(getWidth(), getHeight());
	glVertex2f(0, getHeight());
	glEnd();

	glPopMatrix();

	fontPrinter->print(getFontId(), getPositionX(), getPositionY(),
			this->getCurrentTextColor(), this->getString());
}

template<class C, typename T>
void CGUIListOption<C, T>::mouseButtons(int button, int state, int x, int y) {
	if (this->isHover()) {
		if (button == GLUT_LEFT && state == GLUT_DOWN) {
			if (m_parent != NULL && m_func != NULL) {
				//cout << "CLICK on ID" << m_ID << " - " << getString() << endl;
				(this->m_parent->*m_func)(m_ID);
			}
		}
	}
}

#endif
