/** @file GUIMenu.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CGUIMenu
 *
 *
 * \brief GUI menu
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CGUIMENU_H_
#define _CGUIMENU_H_

#define	ALIGN_LEFT		0
#define ALIGN_RIGHT		1
#define ALIGN_CENTER 	2

#include "GameObject2D.h"
#include "Collection.h"
#include "GUIMenuItem.h"

template<class T>
class CGUIMenu: public CCollection<T> , public CGUIElement {
public:
	CGUIMenu();
	virtual ~CGUIMenu();

	void render();
	void update();

	void mouseButtons(int button, int state, int x, int y);
	void mouseMove(int x, int y);

	//

	void setFontId(int id) {
		m_fontId = id;
		m_optimized = false;
	}
	;
	void getFontId() {
		return m_fontId;
		m_optimized = false;
	}

	void setAlign(int align) {
		m_align = align;
	}
	;

protected:

	void optimize();

	bool m_optimized;

	int m_fontId;
	int m_align;

};

template<class T>
CGUIMenu<T>::CGUIMenu() {
	m_optimized = false;
	m_align = ALIGN_LEFT;

	m_paddingLeft = 10;
	m_paddingTop = 10;
	m_lineHeight = 22;

	this->setBackgroundColor(0, 0, 0, 128);
	this->setBackgroundHoverColor(255, 0, 0, 255);

}

template<class T>
CGUIMenu<T>::~CGUIMenu() {
	// TODO smazat vsechny prvky menu
}

template<class T>
void CGUIMenu<T>::render() {

	GLfloat px = 0;
	T* curr = NULL;

	glPushMatrix();

	glTranslatef(getPositionX(), getPositionY(), 0);

	Color4f backgroundColor = this->getCurrentBackgroundColor();

	glColor4f(backgroundColor.r, backgroundColor.g, backgroundColor.b,
			backgroundColor.alpha);

	glBegin(GL_QUADS);
	glVertex2f(0, 0);
	glVertex2f(getWidth(), 0);
	glVertex2f(getWidth(), getHeight());
	glVertex2f(0, getHeight());
	glEnd();

	glPopMatrix();

	for (int i = 0; i < this->size(); i++) {

		curr = this->get(i);
		if (curr != NULL) {
			if (curr->isVisible()) {

				switch (m_align) {
				case ALIGN_LEFT:
					px = getPositionX();

					break;

				case ALIGN_RIGHT:
					px = getPositionX() + getWidth() - curr->getString().size();

					cout << curr->getString().size() << endl;

					break;
				case ALIGN_CENTER:
					break;
				}

				curr->setPosition(getPaddingLeft() + px,
						getPaddingTop() + getPositionY() + i * m_lineHeight);

				curr->render();
			}
		}
	}

}

template<class T>
void CGUIMenu<T>::mouseButtons(int button, int state, int x, int y) {

	CGameObject2D* curr = NULL;

	for (int i = 0; i < this->size(); i++) {

		curr = this->get(i);
		if (curr != NULL) {
			if (curr->isVisible()) {
				curr->mouseButtons(button, state, x, y);
			}
		}
	}
}

template<class T>
void CGUIMenu<T>::mouseMove(int x, int y) {

	CGameObject2D* curr = NULL;

	for (int i = 0; i < this->size(); i++) {

		curr = this->get(i);
		if (curr != NULL) {
			if (curr->isVisible()) {
				curr->mouseMove(x, y);
			}
		}
	}
}

template<class T>
void CGUIMenu<T>::update() {

	CGameObject2D* curr = NULL;

	for (int i = 0; i < this->size(); i++) {

		curr = this->get(i);
		if (curr != NULL) {
			if (curr->isVisible()) {
				curr->update();
			}
		}
	}
}

template<class T>
void CGUIMenu<T>::optimize() {

	CGUIMenuItem<T>* curr = NULL;

	for (int i = 0; i < this->size(); i++) {

		curr = this->get(i);
		if (curr != NULL) {
			curr->setPosition(getPositionX(), getPositionY() + i * m_lineHeight);

		}
	}

}

#endif /* GUIMENU_H_ */
