/** @file GUIMessage.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CGUIMessage
 *
 *
 * \brief GUI zpráva
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CGUIMESSAGE_H_
#define _CGUIMESSAGE_H_

#include "GUIFrame.h"

template<class T>
class CGUIMessage: public CGUIFrame {
public:

	CGUIMessage();
	CGUIMessage(const char* message);
	CGUIMessage(string message);

	virtual ~CGUIMessage();
	void init(string message);

	void setMessage(string message) {
		this->get(0)->setString(message);
	}
	;

	void callOnConfirm();

	void setCallback(T* _instance, void(T::*_func)(void)) {
		m_instance = _instance;
		m_funcOnConfirm = _func;
	}

	virtual void centered();

protected:
	T* m_instance;
	void (T::*m_funcOnConfirm)(void);

};

template<class T>
CGUIMessage<T>::CGUIMessage() {
	init("No message set.");
}

template<class T>
CGUIMessage<T>::CGUIMessage(string message) {
	init(message);
}

template<class T>
CGUIMessage<T>::CGUIMessage(const char * message) {
	string tmp(message);
	init(tmp);
}

template<class T>
CGUIMessage<T>::~CGUIMessage() {
	// TODO Auto-generated destructor stub
}

template<class T>
void CGUIMessage<T>::init(string message) {

	this->setColor(0, 0, 0, 128);
	this->setDimension(300, 150);
	this->setPosition(100, 100);

	CGUIButton<CGUIMessage>* button = new CGUIButton<CGUIMessage> ;
	button->setString("OK");
	button->setDimension(30, 20);

	button->setCallbackClass(this);
	button->setMouseDownCallback(&CGUIMessage::callOnConfirm);

	CGUILabel* label = new CGUILabel();
	label->setString(message);

	this->addChild(label);
	this->addChild(button);
}

template<class T>
void CGUIMessage<T>::callOnConfirm() {

	if (this->m_instance != NULL && this->m_funcOnConfirm != NULL) {
		(this->m_instance->*this->m_funcOnConfirm)();
	}

}

template<class T>
void CGUIMessage<T>::centered() {
	GLint viewport[4]; // informace o viewportu
	glGetIntegerv(GL_VIEWPORT, viewport); // ziskame informace o VP

	GLfloat positionX = (viewport[2] / 2) - (this->getWidth() / 2);
	GLfloat positionY = (viewport[3] / 2) - (this->getHeight() / 2);

	setPosition(positionX, positionY);

	CGUIElement* curr = NULL;

	for (int i = 1; i < size(); i++) {
		if ((curr = getChild(i)) != NULL) {
			GLfloat px = positionX + (getWidth() / 2) - curr->getWidth() / 2;
			curr->setPositionX(px);
		}
	}
}

#endif /* GUIMESSAGE_H_ */
