/** @file GUIPromptList.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CGUIPromptList
 *
 *
 * \brief GUI výzva - výběr z listu
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef CGUICONFIRMLIST_H_
#define CGUICONFIRMLIST_H_

#include "GUIPrompt.h"
#include "GUIList.h"

template<class T>
class CGUIPromptList: public CGUIPrompt<T> {
public:
	CGUIPromptList();
	virtual ~CGUIPromptList();
	virtual void initChilds();

	int getValue();

	void addOption(int value, string label);
	void centered();
};

template<class T>
CGUIPromptList<T>::CGUIPromptList() {
	// TODO Auto-generated constructor stub

}

template<class T>
CGUIPromptList<T>::~CGUIPromptList() {
	// TODO Auto-generated destructor stub
}

template<class T>
void CGUIPromptList<T>::initChilds() {

	CGUILabel* text = new CGUILabel();
	CGUIList<int>* list = new CGUIList<int> ();
	list->addOption(-1, "-- NEW --");
	list->centered();

	CGUIButton<CGUIPromptList> *buttonOK =
			new CGUIButton<CGUIPromptList> (this);
	buttonOK->setMouseDownCallback(&CGUIPromptList::onConfirm);
	buttonOK->setString("OK");
	buttonOK->setDimension(25, 14);

	CGUIButton<CGUIPromptList> *buttonCancel = new CGUIButton<CGUIPromptList> (
			this);
	buttonCancel->setMouseDownCallback(&CGUIPromptList::onCancel);
	buttonCancel->setString("CANCEL");
	buttonCancel->setDimension(50, 14);

	this->addChild(text);
	this->addChild(list);
	this->addChild(buttonOK);
	this->addChild(buttonCancel);

}

template<class T>
void CGUIPromptList<T>::addOption(int value, string label) {
	CGUIElement* list = this->getChild(1);
	if (list != NULL) {
		dynamic_cast<CGUIList<int>*> (list)->addOption(value, label);
	}
}

template<class T>
int CGUIPromptList<T>::getValue() {
	CGUIElement* list = NULL;

	if ((list = this->getChild(1)) != NULL) {
		return dynamic_cast<CGUIList<int>*> (list)->getActiveValue();
	}

	return -2;
}

template<class T>
void CGUIPromptList<T>::centered() {
	CGUIConfirm<T>::centered();

	CGUIElement* list = this->getChild(1);
	CGUIElement* buttonOK = this->getChild(2);
	CGUIElement* buttonCancel = this->getChild(3);

	float positionY = this->getPositionY();
	float visible = 0;

	if (list != NULL) {
		positionY = list->getPositionY();
		visible = dynamic_cast<CGUIList<int>*> (list)->getCountVisible();
	}

	if (buttonOK != NULL) {
		buttonOK->setPositionY(positionY + visible * this->getLineHeight());
	}

	if (buttonCancel != NULL) {
		buttonCancel->setPositionY(
				positionY + (visible + 1) * this->getLineHeight());
	}

}

#endif /* CGUICONFIRMLIST_H_ */
