/** @file GameState.h
 *
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CGameState
 *
 *
 * \brief Stav hry
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 *
 */

#ifndef _CGAMESTATE_H_
#define _CGAMESTATE_H_

#include "TextureManager.h"
#include "Renderer.h"
#include "GameObject3DManager.h"
#include "GameObject2DManager.h"
#include "Camera.h"
#include "Font.h"

#include "GUIHeaders.h"

#include "Mouseable.h"
#include "KeyListener.h"

class CGameState: public CMouseable, public CKeyListener {

public:

	virtual ~CGameState(){
		if( m_referencesObject2D != NULL ){
			delete [] m_referencesObject2D;
			m_referencesObject2D = NULL;
		}
	}

	virtual void init(void) = 0;
	virtual void clear(void) = 0;
	virtual void updateScene(void) = 0;

	void update() {

		updateScene();

		if (m_gameObject3DManager != NULL) {
			m_gameObject3DManager->update();
		}

		if (m_gameObject2DManager != NULL) {
			m_gameObject2DManager->update();
		}
	}

	void render(void) {
		if (initialized && m_running) {
			m_renderer -> render();
		}
	}

	void keyboard(unsigned char key, int x, int y) {
		if (m_gameObject3DManager != NULL) {
			m_gameObject3DManager->keyboard(key, x, y);
		}

		if (m_gameObject2DManager != NULL) {
			m_gameObject2DManager->keyboard(key, x, y);
		}
	}
	;

	void keyboardSpecial(int key, int x, int y) {
		if (m_gameObject3DManager != NULL) {
			m_gameObject3DManager->keyboardSpecial(key, x, y);
		}

		if (m_gameObject2DManager != NULL) {
			m_gameObject2DManager->keyboardSpecial(key, x, y);
		}
	}

	void mouseMove(int x, int y) {
		if (m_gameObject3DManager != NULL) {
			m_gameObject3DManager->mouseMove(x, y);
		}

		if (m_gameObject2DManager != NULL) {
			m_gameObject2DManager->mouseMove(x, y);
		}
	}

	void mouseButtons(int button, int state, int x, int y) {
		if (m_gameObject3DManager != NULL) {
			m_gameObject3DManager->mouseButtons(button, state, x, y);
		}

		if (m_gameObject2DManager != NULL) {
			m_gameObject2DManager->mouseButtons(button, state, x, y);
		}
	}

	/**
	 * Zobrazi 2D objekt ve scene
	 * @param id
	 */

	void showSceneObject2D(int id) {
		CGameObject2D* object = this->getReferencedObject2D(id);
		if (object != NULL) {
			object->show();
		}
	}

	/**
	 * Schova 2D objekt ve scene
	 * @param id
	 */

	void hideSceneObject2D(int id) {
		CGameObject2D* object = this->getReferencedObject2D(id);
		if (object != NULL) {
			object->hide();
		}
	}

	/**
	 * Vrati odkaz na objekt
	 * @param index
	 * @return
	 */
	CGameObject2D* getReferencedObject2D(int index) {

		if (index < 0 || m_referencesObject2DSize <= index ) {
			return NULL;
		}

		CGameObject2D* object = NULL;
		if ((object = this->m_gameObject2DManager->get(
				m_referencesObject2D[index])) != NULL) {
			return object;
		}

		return NULL;
	}

	/**
	 * Vytvori pole pro odkazy na refernce hry
	 * @param size
	 */

	void createObject2DReferences( int size ){
		if( this->m_referencesObject2D == NULL){

			this->m_referencesObject2D = new int[size];
			this->m_referencesObject2DSize = size;

			for( int i = 0; i < m_referencesObject2DSize; i++ ){
				this->m_referencesObject2D[i] = -1;
			}

		}
	}

	bool isRunning() {
		return m_running;
	}

	int getNextState() {
		return m_nextStateId;
	}

	void setNextState(int nextStateId) {
		m_nextStateId = nextStateId;
	}

protected:
	CGameState() :
		initialized(false), m_nextStateId(-1) {

		m_renderer = CRenderer::getInstance(); 						 // 	nastaveni renderu
		m_gameObject3DManager = CGameObject3DManager::getInstance(); //	nastaveni spravce objektu hry
		m_gameObject2DManager = CGameObject2DManager::getInstance();
		m_camera = CCamera::getInstance(); 							//  kamera

		m_renderer->setObject3DManager(m_gameObject3DManager);
		m_renderer->setObject2DManager(m_gameObject2DManager);

		m_referencesObject2D = NULL;

	}

	bool initialized; // priznak inicializace - udava, zdali je scena jiz inicializovana
	bool m_running;
	int m_nextStateId;

	int* m_referencesObject2D;
	int	 m_referencesObject2DSize;

	CRenderer* m_renderer; // vykreslovac
	CGameObject3DManager* m_gameObject3DManager; // spravce objektu
	CGameObject2DManager* m_gameObject2DManager;
	CCamera* m_camera; // kamera

};

#endif

