#include "ImageLoader.h"

/**
 * Konstruktor
 */
CImageLoader::CImageLoader(){
	m_data = 0;
}

/**
 * Destruktor
 */
CImageLoader::~CImageLoader(){
	clear();							// dealkoce prostředků
}

/**
 * Načtení obrázku ze souboru
 * @param fileName
 * @return
 */
bool CImageLoader::loadImage( const char *fileName ){

	if( m_data != 0 ){
		clear();
	}

	ilGenImages(1, &m_data);											// vytvorime ID obrazku
	ilBindImage(m_data);

	cout << "Image " << fileName << " ";

	if( ilLoadImage( fileName ) ){											// nacteni obrazku probehlo v poradku
		cout << " ... success" << endl;										// vypiseme, ze jsme texturu nacetli

		ILinfo imageInfo;													// informace o obrazku
		iluGetImageInfo(&imageInfo);										// ulozime informace

		if( imageInfo . Origin == IL_ORIGIN_UPPER_LEFT ){					// v pripade, ze je obrazek nacten zrcadlove otocen
			iluFlipImage();													// otocime jej
		}

		if( !ilConvertImage( IL_RGB, IL_UNSIGNED_BYTE ) ){					// prevedeme obrazek na byty, abychom s nim mohli hezky pracovat
			cout << "error [conversion]: " << iluErrorString( ilGetError() ) << endl;
		}
	}else{																	// vyskytla se chyba
		cout << " ... error: " << iluErrorString(ilGetError()) << endl;	// vypiseme informaci o chybe
		return false;
	}

	return true;
}

/**
 * Vrátí informace o obrázku
 * @return
 */
CImageLoader::ImageInfo CImageLoader::getInfo ( ){

	ImageInfo info;

	if( m_data == 0 ){
		return info;
	}

	info . width 	= ilGetInteger(IL_IMAGE_WIDTH);			// šířka obrázku
	info . height 	= ilGetInteger(IL_IMAGE_HEIGHT);		// výška obrázku
	info . depth	= ilGetInteger(IL_IMAGE_DEPTH);			// barevná hloubka
	info . format 	= ilGetInteger(IL_IMAGE_FORMAT);		// formát
	info . data 	= ilGetData();							// data obrázku

	return info;
}

/**
 * Uvolnění prostředků
 */
void CImageLoader::clear( void ){
	if( m_data != 0 ){
		ilDeleteImages( 1, &m_data );
		m_data = 0;
	}
}


