#include "Level.h"

#include "Cube.h"

CLevel::CLevel() {
	m_map = NULL;
	m_mapWidth = 0;
	m_mapLength = 0;
	m_countTargets = 0;
	m_actorTurns = 0;
	m_finished = false;

	this->m_objects.m_pBox = new CGOCube;
	this->m_objects.m_pBox->setDimension(0.5, 0.5, 0.5);

	this->m_objects.m_pTarget = new CGOPlane;
	this->m_objects.m_pTarget->setColor(255, 0, 0, 255);

	this->m_objects.m_pCrate = new CGOCube;

	this->m_objects.m_actor.object = new CGOCube;
	this->m_objects.m_actor.positionX = 0;
	this->m_objects.m_actor.positionZ = 0;
	this->m_objects.m_actor.object->setColor(0, 0, 255, 255);
}

CLevel::~CLevel() {

	// smazani objektu

	if (m_objects.m_pBox != NULL) {
		delete m_objects.m_pBox;
	}

	if (m_objects.m_actor.object != NULL) {
		delete m_objects.m_actor.object;
	}

	if (m_objects.m_pCrate != NULL) {
		delete m_objects.m_pCrate;
	}

	if (m_objects.m_pTarget != NULL) {
		delete m_objects.m_pTarget;
	}

	// smazani mapy

	this->clearMap();

}

void CLevel::render() {

	/////////////////////////////////////////////////////////////
	// vykresleni mistnosti
	walls[0].setPosition(getPositionX(), 0, getPositionZ());
	walls[0].setDimension(getWidth(), getHeight(), getLength());

	//predni stena

	walls[1].setPosition(getPositionX(), 0, -1 * getPositionZ());
	walls[1].setRotation(-90, 0, 90);
	walls[1].setDimension(getLength(), getHeight(), getHeight());

	// prava stena
	walls[2].setPosition(getPositionX(), 0, getPositionZ());
	walls[2].setRotation(-90, 0, 0);
	walls[2].setDimension(getWidth(), getHeight(), getHeight());

	walls[3].setPosition(-1 * getPositionX(), 0, -1 * getPositionZ());
	walls[3].setRotation(-90, 0, 90);
	walls[3].setDimension(getLength(), getHeight(), getHeight());

	//zadni stena
	walls[4].setPosition(getPositionX(), 0, -1 * getPositionZ());
	walls[4].setRotation(-90, 0, 0);
	walls[4].setDimension(getWidth(), getHeight(), getHeight());

	for (int i = 0; i < 5; i++) {
		walls[i].render();
	}

	/////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////

	float nHalfWidth = (m_mapWidth / 2) * -1;
	float nHalfLength = (m_mapLength / 2) * -1;

	for (int i = 0; i < m_mapLength; i++) {
		for (int j = 0; j < m_mapWidth; j++) {
			switch (m_map[i][j]) {

			case LO_WALL:
				m_objects.m_pBox->setPosition(nHalfWidth + j + 0.25, 0,
						nHalfLength + i + 0.25);
				m_objects.m_pBox->render();
				break;

			case LO_CRATE:
				m_objects.m_pCrate->setColor(255, 255, 255, 255);
				m_objects.m_pCrate->setPosition(nHalfWidth + j, 0,
						nHalfLength + i);
				m_objects.m_pCrate->render();
				break;

			case LO_TARGET:
				m_objects.m_pTarget->setPosition(nHalfWidth + j, 0.01,
						nHalfLength + i);
				m_objects.m_pTarget->render();
				break;

			case LO_CRATE_TARGET:
				m_objects.m_pCrate->setPosition(nHalfWidth + j, 0.01,
						nHalfLength + i);
				m_objects.m_pCrate->setColor(0, 255, 0, 255);
				m_objects.m_pCrate->render();
				break;

			}

		}
	}

	// render player
	m_objects.m_actor.object->setPosition(nHalfWidth + getPlayerPositionX(), 0,
			nHalfLength + getPlayerPositionY());

	m_objects.m_actor.object->render();

}

/**
 * Vytvoření levelu
 * @param map
 * @param width
 * @param length
 */

void CLevel::createMap(int** map, int width, int length, int playerPositionX,
		int playerPositionY) {

	this->clearMap();

	m_mapLength = length;
	m_mapWidth = width;
	m_countTargets = 0;
	m_finished = false;
	m_actorTurns = 0;

	setDimension(width + 8, 5, length + 8);
	setPosition((getWidth() / 2) * -1, 0, (getLength() / 2) * -1);

	m_map = new int*[length];

	for (int i = 0; i < length; i++) {
		m_map[i] = new int[width];
		for (int j = 0; j < width; j++) {
			m_map[i][j] = 0;
		}
	}

	for (int i = 0; i < length; i++) {
		for (int j = 0; j < width; j++) {
			m_map[i][j] = map[i][j];

			if (m_map[i][j] == LO_TARGET) {
				this->m_countTargets++;
			}

		}
	}

	setPlayerPosition(playerPositionX, playerPositionY);

}

/**
 * Nastavení textury pro krabice
 * @param texId
 */
void CLevel::setCrateTexId(int texId) {
	m_objects.m_pCrate->setTextureId(texId);
	m_objects.m_actor.object->setTextureId(texId);
}

/**
 * Nastavení textury pro boxy
 * @param texId
 */
void CLevel::setBoxTexId(int texId) {
	m_objects.m_pBox->setTextureId(texId);
}

/**
 * Pohyb hrace
 * @param x
 * @param y
 * @return
 */
bool CLevel::moveActorTo(int x, int y) {

	if (!m_finished) {

		if (!(m_objects.m_actor.positionX + x >= 0
				&& m_objects.m_actor.positionX + x < m_mapWidth
				&& m_objects.m_actor.positionZ + y >= 0
				&& m_objects.m_actor.positionZ + y < m_mapLength)) {

			return false;
		}

		int nextPosX = m_objects.m_actor.positionX + x;
		int nextPosZ = m_objects.m_actor.positionZ + y;

		if (m_map[nextPosZ][nextPosX] == LO_WALL) {
			return false;
		}

		if (m_map[nextPosZ][nextPosX] == LO_CRATE || m_map[nextPosZ][nextPosX]
				== LO_CRATE_TARGET) {

			if (!(m_objects.m_actor.positionX + x * 2 >= 0
					&& m_objects.m_actor.positionX + x * 2 < m_mapWidth
					&& m_objects.m_actor.positionZ + y * 2 >= 0
					&& m_objects.m_actor.positionZ + y * 2 < m_mapLength)) {

				return false;
			}

			if (m_map[nextPosZ + y][nextPosX + x] == LO_WAY || m_map[nextPosZ
					+ y][nextPosX + x] == LO_TARGET) {

				cout << m_map[nextPosZ + y][nextPosX + x] << endl;

				if (m_map[nextPosZ][nextPosX] == LO_CRATE) {
					m_map[nextPosZ][nextPosX] = LO_WAY;
				} else {

					if (m_map[nextPosZ][nextPosX] == LO_CRATE_TARGET) {
						m_countTargets++;
					}

					m_map[nextPosZ][nextPosX] = LO_TARGET;
				}

				if (m_map[nextPosZ + y][nextPosX + x] == LO_WAY) {
					m_map[nextPosZ + y][nextPosX + x] = LO_CRATE;
				} else {
					m_map[nextPosZ + y][nextPosX + x] = LO_CRATE_TARGET;
					m_countTargets--;
				}

			} else {
				return false;
			}

		}

		setPlayerPosition(nextPosX, nextPosZ);

		if (m_countTargets == 0) {
			m_finished = true;
		}

		m_actorTurns++;
	}

	return true;
}

void CLevel::setPlayerPosition(int x, int z) {
	m_objects.m_actor.positionX = x;
	m_objects.m_actor.positionZ = z;
}

void CLevel::clearMap(){

	if( this->m_map != NULL ){

		cout << "DELETE OLD LEVEL" << endl;

		for( int i = 0; i < m_mapLength; i++ ){
			if( m_map[i] != NULL ){
				delete [] m_map[i];
				m_map[i] = NULL;
			}
		}

		delete [] m_map;

		m_map = NULL;
	}
}
