/** @file Level.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CLevel
 *
 *
 * \brief Mapa hry
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _HCLEVEL_
#define _HCLEVEL_

#define LO_VACUUM		0		// nic
#define LO_WALL			1		// zed
#define LO_WAY			2		// pruchodna cesta
#define LO_TARGET		3		// cil
#define LO_CRATE		4		// bedna
#define LO_START		5		// startovni pozice
#define LO_CRATE_TARGET	6		// box v cili
#include "GameObject3D.h"
#include "Plane.h"

class CLevel: public CGameObject3D {

public:

	typedef struct SActor {
		CGameObject3D* object;
		int positionX;
		int positionZ;
	} SACTOR;

	typedef struct SLevelObjects {
		CGameObject3D* m_pTarget;
		CGameObject3D* m_pBox;
		CGameObject3D* m_pCrate;
		SActor m_actor;
	} SLEVELOBJECTS;

	CLevel();
	~CLevel();

	void render();
	void update() {
	}
	;

	void setWallTexId(int num, int texId) {
		if (num < 6) {
			walls[num].setTextureId(texId);
		}
	}
	;
	void setCrateTexId(int texId);
	void setBoxTexId(int texId);
	void setDoorsTex(int texId) {
		doors.setTextureId(texId);
	}
	;
	void setWindow(int texId) {
		window.setTextureId(texId);
	}
	;

	void createMap(int** map, int width, int length, int playerPositionX,
			int playerPositionZ);

	bool moveActorTo(int x, int y);
	bool isFinished() {
		return m_finished;
	}
	;

	void clearMap();

	/**
	 *
	 */
	int getActorTurns() const {
		return m_actorTurns;
	}

private:

	void setPlayerPosition(int x, int y);
	int getPlayerPositionX() {
		return m_objects.m_actor.positionX;
	}
	;
	int getPlayerPositionY() {
		return m_objects.m_actor.positionZ;
	}
	;

	string m_levelName;

	// graficka reprezentace
	CGOPlane walls[6]; // steny
	CGOPlane doors; // dvere
	CGOPlane window; // okno

	int** m_map; // mapa

	int m_mapWidth;
	int m_mapLength;
	int m_countTargets;
	bool m_finished;

	SACTOR m_pActor;
	int m_actorTurns;

	SLevelObjects m_objects;

};

#endif

