/*
 * ProfileManager.cpp
 *
 *  Created on: May 23, 2011
 *      Author: franta
 */

#include <sstream>
#include "ProfileManager.h"

CProfileManager::CProfileManager() {
	m_currentProfile = -1;
}

CProfileManager::~CProfileManager() {
	saveProfiles();
}

void CProfileManager::loadProfiles() {

	ifstream input;
	input.open(FILE_PROFILES, ios::in);
	string line;

	if (!input.is_open()) {
		cerr << "Could not open profiles data." << endl;
		// TODO create new profiles data - sorry!
	}

	// dokud budou radky v souboru
	while (input.good()) {
		getline(input, line); // nacteme novou radku
		istringstream lineData(line, istringstream::in); // do streamu nastavime nacteny radek

		if (input . eof())
			break;

		// prekopirujeme data do struktury
		SPlayerInfo* playerInfo = new SPlayerInfo;

		lineData >> playerInfo->m_id; // id hrace
		lineData >> playerInfo->m_level; // uroven hrace
		lineData >> playerInfo->m_score; // skore hrace
		lineData >> playerInfo->m_name; // jmeno hrace

		cout << line << endl;

		this->m_players.add(playerInfo);
	}

	input.close();

	cout << "Player profiles loaded." << endl;
}

void CProfileManager::saveProfiles() {
	ofstream output;
	output.open(FILE_PROFILES, ios::out); // otevreme soubor s uzivatelskymi ucty

	if (!output.is_open()) { // nelze-li otevrit
		cerr << "Could not open profiles data." << endl; // vypiseme chybu
		// TODO create new profiles data - sorry!
	}

	SPlayerInfo* currPlayer = NULL;
	int size = m_players.size();
	string out;

	for (int i = 0; i < size; i++) { // postupne prochazime hrace
		if ((currPlayer = m_players.get(i)) != NULL) {
			output << currPlayer->m_id << " " << currPlayer->m_level << " " << currPlayer->m_score << " " << currPlayer->m_name << endl;
		}
	}

	output.close(); // zavreme soubor
}

void CProfileManager::init() {
	loadProfiles(); // nacteme uzivatelske profily
}

void CProfileManager::resetPlayerGame() {
	if (isSetProfile()) { // je-li aktivni uzivatelsky ucet
		m_players.get(m_currentProfile)->m_level = 0; // pak mu nastavime uroven na 0
		m_players.get(m_currentProfile)->m_score = 0;
		saveProfiles(); // ulozime profily
	}
}

bool CProfileManager::setActiveProfile(int id) {

	SPlayerInfo* currPlayer = NULL;

	for (int i = 0; i < m_players.size(); i++) { // postupne prochazime hrace
		if ((currPlayer = m_players.get(i)) != NULL) {
			if (currPlayer->m_id == id) { // najdeme-li hrace s prislusnym ID

				cout << "NASTAVEN PROFILE " << id << endl;

				this->m_currentProfile = i; // nastavime jej jako aktivniho
				return true; // ukoncime hledani
			}
		}
	}

	return false;
}

int CProfileManager::createNewPlayer(string name) {

	int maxID = 0; // maximalni identifikator
	SPlayerInfo* currPlayer = NULL;

	for (int i = 0; i < m_players.size(); i++) { // postupne prochazime hrace
		if ((currPlayer = m_players.get(i)) != NULL) {
			if (currPlayer->m_id > maxID) { // v pripade, ze nektery ucet ma vetsi id nez je dosavadni maximalni id
				maxID = currPlayer->m_id; // nastavime toto id jako maximalni
			}
		}
	}
	maxID++;

	SPlayerInfo* newPlayer = new SPlayerInfo; // vytvorime noveho hrace
	newPlayer->m_name = name; // nastavime hraci jmeno
	newPlayer->m_id = maxID;
	newPlayer->m_level = 0;
	newPlayer->m_score = 0;
	m_players.add(newPlayer); // pridame jej do seznamu hracu

	saveProfiles(); // ulozime profily

	return maxID;
}

string CProfileManager::getActiveProfileName() {
	if (isSetProfile()) {
		return m_players.get(m_currentProfile)->m_name;
	}

	return NULL;
}

int CProfileManager::getActiveProfileLevel() {
	if (isSetProfile()) {
		return m_players.get(m_currentProfile)->m_level;
	}

	return -1;
}

int CProfileManager::getActiveProfileScore() {
	if (isSetProfile()) {
		return m_players.get(m_currentProfile)->m_score;
	}

	return -1;
}

void CProfileManager::setActiveProfileLevelUp(){
	if(isSetProfile()){
		m_players.get(m_currentProfile)->m_level += 1;
		cout << "Nastaveni levelu na: " << m_players.get(m_currentProfile)->m_level << endl;
	}

	saveProfiles();
}

void CProfileManager::addActiveProfileScore( int score ){
	if( isSetProfile() ){
		m_players.get(m_currentProfile)->m_score += score;
	}
}



