/** @file ProfileManager.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CProfileManager
 *
 *
 * \brief Správce profilů hráčů
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CPROFILEMANAGER_H_
#define _CPROFILEMANAGER_H_

#define FILE_PROFILES		"data/profiles/profiles.dat"

#include "main.h"
#include "Profile.h"
#include "Singleton.h"
#include "Collection.h"

class CProfileManager: public CSingleton<CProfileManager> {

public:

	/**
	 * Informace o hraci
	 */

	struct SPlayerInfo {

		/**
		 * Jmeno hrace
		 */
		string m_name;

		/**
		 * ID
		 */
		int m_id;

		/**
		 * Uroven
		 */
		int m_level;

		/**
		 * Dosazene skore
		 */
		int m_score;

		/**
		 * Implicitni konstruktor
		 */
		SPlayerInfo() :
			m_id(0), m_level(0), m_score(0) {
		}
		;

	};

	/**
	 * Konstruktor
	 */
	CProfileManager();

	/**
	 * Destruktor
	 */
	virtual ~CProfileManager();

	/**
	 * Načte z datového souboru profily hráčů
	 */
	void loadProfiles();

	/**
	 * Uloží do datového souboru profily hráčů
	 */
	void saveProfiles();

	/**
	 * Načte hráče dle identifikátoru
	 * @param ID
	 * @return
	 */

	int count() {
		return m_players.size();
	}
	;

	/**
	 * Resetuje dosavadní výkony hráče
	 * @return
	 */

	void resetPlayerGame();

	/**
	 * Je nastaven aktivní profil hráče?
	 * @return
	 */
	bool isSetProfile() {
		return m_currentProfile >= 0 ? true : false;
	}
	;

	/**
	 * Inicializace správce profilů
	 */
	void init();

	/**
	 * Nastaví aktivní uživatelský účet
	 * @param id
	 */
	bool setActiveProfile(int id);

	/**
	 * Vytvoří nový uživatelský profil
	 * @param name
	 * @return
	 */
	int createNewPlayer(string name);

	string getActiveProfileName();
	int getActiveProfileLevel();
	int getActiveProfileScore();

	void addActiveProfileScore(int score);

	void setActiveProfileLevelUp();

	SPlayerInfo* getPlayerProfile(int index) {
		return m_players.get(index);
	}

protected:
	int m_currentProfile;
	CCollection<SPlayerInfo> m_players;
};

#endif /* PROFILEMANAGER_H_ */
