#include "Renderer.h"
#include "Font.h"
#include "FontManager.h"

CRenderer* CRenderer::m_instance = NULL;

/**	Konstruktor
 */
CRenderer::CRenderer(void) {

	m_pCamera = NULL;

	this->setBackgroundColor(0, 0, 0, 255);

	setViewportWidth(0);
	setViewportHeight(0);

	blendingDisable();

}

/**
 * Konstruktor s parametry
 * @param viewportWidth
 * @param viewportHeight
 */
CRenderer::CRenderer(int viewportWidth, int viewportHeight) {
	setViewportWidth(viewportWidth);
	setViewportHeight(viewportHeight);

	blendingDisable();
}

/**	Destruktor
 */
CRenderer::~CRenderer(void) {

	if (m_instance != NULL)
		delete m_instance;

}

/**	Vykreslí objekty
 */
void CRenderer::render(void) {

	if (m_pObject3DManager == NULL && m_pObject2DManager == NULL) {
		cout << "Error: Object manager is not connected to renderer." << endl;
	}

	// nastaveni parametru vykreslovani

	//blending
	if( m_blending ){
		glEnable(GL_BLEND);
		glBlendFunc(m_blendingSourceFactor, m_blendingDestinationFactor);
	}else{
		glDisable(GL_BLEND);
	}

	// Clear Color and Depth Buffers
	glClearColor(m_backgroundColor.r, m_backgroundColor.g, m_backgroundColor.b, m_backgroundColor.alpha);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Reset transformations
	glLoadIdentity();

	m_pCamera -> look();


	// vykresleni 3d objektu
	if (m_pObject3DManager != NULL) {
		CGameObject3D* curr = NULL;

		int objectsCount = m_pObject3DManager->size();
		for (int i = 0; i < objectsCount; i++) {

			curr = m_pObject3DManager->get(i);
			if (curr != NULL) {
				if (curr->isVisible()) {
					curr->beforeRender();
					curr->render();
					curr->afterRender();
				}
			}
		}
	}

	glPopMatrix();

	// vykresleni 3d objektu
	if (m_pObject2DManager != NULL) {

		enable2D();

		CGameObject2D* curr = NULL;

		//cout << "2D Object rendering" << endl;
		int objectsCount = m_pObject2DManager->size();

		for (int i = 0; i < objectsCount; i++) {

			curr = m_pObject2DManager->get(i);
			if (curr != NULL) {
				if (curr->isVisible()) {
					curr->render();

				}
			}

		}

		disable2D();
	}

	glutSwapBuffers(); // prohozeni bufferu
}

/**
 * Nastaví projekční matici na 2D vykreslování
 */
void CRenderer::enable2D() {

	GLint viewport[4]; // informace o viewportu
	glGetIntegerv(GL_VIEWPORT, viewport); // ziskame informace o VP

	glMatrixMode(GL_PROJECTION); // nastavime projekcni matici
	glPushMatrix(); // ulozime ji na zasobnik

	glLoadIdentity(); // reset projekcni matice

	glOrtho(viewport[0], viewport[0] + viewport[2], viewport[1] + viewport[3],
			viewport[1], -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glLoadIdentity();

	glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_LIGHTING_BIT); // priznaky na zasobnik
	glDisable(GL_DEPTH_TEST); 							// vypneme hloubkove testovani
	glDisable(GL_LIGHTING); 							// vypneme svetla

}

/**
 * Vypne projekční matici na 2D vykreslovaní
 */
void CRenderer::disable2D() {

	glPopAttrib();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

}

/* BLENDING */

/**
 * Vypne blending
 */
void CRenderer::blendingDisable(){
	m_blending = false;
	cout << "Blending state: disabled" << endl;
}

/**
 * Zapne blending s defaultním nastavením
 */
void CRenderer::blendingEnable(){
	blendingEnable( GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA);
	cout << "Bleding state: enabled" << endl;
}

/**
 * Zapne blending s parametry
 * @param sfactor
 * @param dfactor
 */
void CRenderer::blendingEnable(GLenum sfactor, GLenum dfactor){
	m_blending = true;
	setBlendingParams( sfactor, dfactor );
}

/**
 * Nastaví parametry blendingu
 * @param sfactor
 * @param dfactor
 */
void CRenderer::setBlendingParams(GLenum sfactor, GLenum dfactor){
	m_blendingSourceFactor = sfactor;
	m_blendingDestinationFactor = dfactor;
}

void CRenderer::disconnect(){
	this->m_pObject2DManager = NULL;
	this->m_pObject3DManager = NULL;
	this->m_pCamera = NULL;
}

