/** @file Renderer.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CRenderer
 *
 *
 * \brief Vykreslovač objektů
 *
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CRENDERER_
#define	_CRENDERER_

#include "main.h"
#include "GameObject3DManager.h"
#include "GameObject2DManager.h"
#include "Camera.h"

class CRenderer: public CSingleton<CRenderer> {

public:

	CRenderer();
	CRenderer(int viewportWidth, int viewportHeight);

	~CRenderer();

	void setViewport(int width, int height) {
		m_viewportWidth = width;
		m_viewportHeight = height;
	}
	;
	void setViewportWidth(int width) {
		m_viewportWidth = width;
	}
	;
	void setViewportHeight(int height) {
		m_viewportHeight = height;
	}
	;

	void setBackgroundColor(Color4f color) {
		m_backgroundColor = color;
	}
	;
	void setBackgroundColor(float red, float green, float blue, float alpha) {
		m_backgroundColor.r = red / 255.0f;
		m_backgroundColor.b = blue / 255.0f;
		m_backgroundColor.g = green / 255.0f;
		m_backgroundColor.alpha = alpha / 255.0f;
	}
	;

	int getViewportWidth() const {
		return m_viewportWidth;
	}
	;
	int getViewportHeight() const {
		return m_viewportHeight;
	}
	;

	void setCamera(CCamera* camera) {
		m_pCamera = camera;
	}
	;
	CCamera* getCamera() const {
		return m_pCamera;
	}

	void setObject3DManager(CGameObject3DManager* object3DManager) {
		m_pObject3DManager = object3DManager;
	}
	;
	void setObject2DManager(CGameObject2DManager* object2DManager) {
		m_pObject2DManager = object2DManager;
	}
	;

	void render(void);

	/* Blending */
	void blendingEnable();
	void blendingEnable(GLenum sfactor, GLenum dfactor);
	void blendingDisable();
	void setBlendingParams(GLenum sfactor, GLenum dfactor);

	void disconnect();

private:

	void enable2D(void);
	void disable2D(void);

	static CRenderer* m_instance;

	int m_viewportWidth; // sirka okna
	int m_viewportHeight; // vyska okna

	Color4f m_backgroundColor;

	CGameObject3DManager* m_pObject3DManager; // manazer 3D objektu
	CGameObject2DManager* m_pObject2DManager; // manazer 2D objektu
	CCamera* m_pCamera; // ukazatel na kameru

	/* Blending */
	// Více na http://www.opengl.org/sdk/docs/man/xhtml/glBlendFunc.xml
	bool m_blending;
	GLenum m_blendingDestinationFactor; // jak bude vypocitana hodnota RGBA cile
	GLenum m_blendingSourceFactor; // jak bude vypocitana hodnota RGBA zdroje

};

#endif

