/** @file Singleton.h
* Hlavičkový soubor Kolekce. Kolekce slouží k ukládání proměnných stejného datového typu.
* \author František Toman
*
* \version 1.0
*
*/


/**
 * \class CSingleton
 *
 *
 * \brief Šablona třídy kolekce
 *
 * Šablona třídy CCollection (kolekce) umožňuje třídám, jež tuto dědí, ukládat proměnné specifického datového typu.
 *
 * Speciální vlastností této kolekce je možnost nastavení zámku v kolekci. V případě, že je zámek nastaven, pak jsou při vyčištění kolekce smazány pouze prvky,
 * na pozici vyšší než je hodnota zámku.
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CSINGLETON_H_
#define _CSINGLETON_H_

#include "main.h"

template <class T>
class CSingleton{

public:

	static T*	getInstance(){				// vraci ukazatel na instanci objektu T

		if( m_instance == NULL ){
			m_instance = new T;
		}
		
		return m_instance;
	};

	static 	void deleteInstance(){
		if( m_instance != NULL ){
			delete m_instance;
			m_instance = NULL;
		}
	}

protected:
	static T* m_instance;					// staticka instance

};

template<class T>
T* CSingleton<T>::m_instance = NULL ;		// inicializace staticke instance

#endif

