/*
 * SokobanSceneMenu.cpp
 *
 *  Created on: May 21, 2011
 *      Author: franta
 */

#include "SokobanGame.h"
#include "SokobanSceneMenu.h"

#include "LevelLoader.h"
#include "Cursor.h"

#include "GUIConfirm.h"
#include "GUIPrompt.h"
#include "GUIPromptList.h"

#include "ProfileManager.h"

CSokobanSceneMenu::CSokobanSceneMenu() {

	/**
	 * Reference:
	 *  	0 - tmave pozadi
	 *  	1 - nadpis sokoban
	 *  	2 - hlavni menu
	 *  	3 - confirm reset hry
	 *  	4 - confirm konec hry?
	 *  	5 - formular novy uzivatel
	 *  	6 - kurzor
	 *  	7 - jmeno hrace
	 *  	8 - uroven hrace
	 *  	9 - skore hrace
	 *  	10 - vyber hrace
	 */

}

CSokobanSceneMenu::~CSokobanSceneMenu() {

}

void CSokobanSceneMenu::init() {

	this->m_running = true; // scena je aktivni
	this->createObject2DReferences(50);

	// ---------------------------------------------------------------------
	// Nastaveni animace

	m_fadeIn = true;
	m_fadeOut = false;
	m_fadesAnimation.reset();
	m_fadesAnimation.setup(true, 0, 150, 2);

	// ---------------------------------------------------------------------
	// ---------------------------------------------------------------------

	// 2D OBJECTS

	// ---------------------------------------------------------------------
	// nadpis

	// na pozadi tmava pruhledna barva
	CGUIImage* img = new CGUIImage();
	img->setColor(0, 0, 0, 255);
	img->setPosition(0, 0);
	img->setDimension(800, 600);

	m_gameObject2DManager->add(img);
	m_referencesObject2D[0] = m_gameObject2DManager->getLastInsertID();

	// nadpis sokoban
	CGUILabel* label = new CGUILabel();
	label->setString("SOKOBAN 3D");
	label->setPosition(100, 60);
	label->setFontId(2);
	label->hide();

	m_gameObject2DManager->add(label);
	m_referencesObject2D[1] = m_gameObject2DManager->getLastInsertID();

	// ---------------------------------------------------------------------
	// ---------------------------------------------------------------------


	// ---------------------------------------------------------------------
	// Vytvoření hlavního menu

	CGUIMenu<CGUIMenuItem<CSokobanSceneMenu> >* mainMenu = new CGUIMenu<
			CGUIMenuItem<CSokobanSceneMenu> > ;
	mainMenu->hide();

	CGUIMenuItem<CSokobanSceneMenu>* miContinue = new CGUIMenuItem<
			CSokobanSceneMenu> ();
	CGUIMenuItem<CSokobanSceneMenu>* miNewGame = new CGUIMenuItem<
			CSokobanSceneMenu> ();

	CGUIMenuItem<CSokobanSceneMenu>* miSelectProfile = new CGUIMenuItem<
			CSokobanSceneMenu> ();
	CGUIMenuItem<CSokobanSceneMenu>* miQuit = new CGUIMenuItem<
			CSokobanSceneMenu> ();

	mainMenu->setPosition(100, 200);

	mainMenu->add(miContinue);
	mainMenu->add(miNewGame);

	mainMenu->add(miSelectProfile);
	mainMenu->add(miQuit);

	miContinue->setString("CONTINUE ... ");
	miNewGame->setString("NEW GAME");

	miSelectProfile->setString("SELECT PROFILE");
	miQuit->setString("QUIT");

	miContinue->setDimension(100, 14);
	miNewGame->setDimension(100, 14);
	miSelectProfile->setDimension(100, 14);
	miQuit->setDimension(100, 14);

	// -- nastavení callbacků položek hlavního menu
	miContinue->setCallback(this, &CSokobanSceneMenu::playGame, NULL);
	miNewGame->setCallback(this, &CSokobanSceneMenu::requestResetGame, NULL);
	miSelectProfile->setCallback(this, &CSokobanSceneMenu::requestSelectPlayer,
			NULL);
	miQuit->setCallback(this, &CSokobanSceneMenu::requestQuitGame, NULL);

	m_gameObject2DManager->add(mainMenu);
	m_referencesObject2D[2] = m_gameObject2DManager->getLastInsertID();

	// ---------------------------------------------------------------------
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Vytvoření potvrzovacích zpráv

	// --- reset hry ---
	CGUIConfirm<CSokobanSceneMenu>* confResetGame = new CGUIConfirm<
			CSokobanSceneMenu> ();
	confResetGame->initChilds();
	confResetGame->setDimension(282, 80);
	confResetGame->setConfirmMessage("WOULD YOU RESET CURRENT GAME?");
	confResetGame->setCallback(this, &CSokobanSceneMenu::resetGame);
	confResetGame->hide();
	confResetGame->centered();

	m_gameObject2DManager->add(confResetGame);
	m_referencesObject2D[3] = m_gameObject2DManager->getLastInsertID();

	// --- konec hry ---

	CGUIConfirm<CSokobanSceneMenu>* confQuitGame = new CGUIConfirm<
			CSokobanSceneMenu> ();
	confQuitGame->initChilds();
	confQuitGame->setConfirmMessage("YOU REALLY WANT TO QUIT?");
	confQuitGame->setDimension(226, 80);
	confQuitGame->setCallback(this, &CSokobanSceneMenu::quitGame);
	confQuitGame->hide();
	confQuitGame->centered();

	m_gameObject2DManager->add(confQuitGame);
	m_referencesObject2D[4] = m_gameObject2DManager->getLastInsertID();

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Vstupy a výběry

	CGUIPrompt<CSokobanSceneMenu>* promptCreatePlayer = new CGUIPrompt<
			CSokobanSceneMenu> ();
	promptCreatePlayer->setDimension(268, 100);
	promptCreatePlayer->initChilds();
	promptCreatePlayer->setMaxChars(10);
	promptCreatePlayer->setConfirmMessage("TYPE YOUR NAME (MAX. 10 CHARS)");
	promptCreatePlayer->setCallback(this, &CSokobanSceneMenu::createNewPlayer);
	promptCreatePlayer->hide();
	promptCreatePlayer->centered();

	m_gameObject2DManager->add(promptCreatePlayer);
	m_referencesObject2D[5] = m_gameObject2DManager->getLastInsertID();

	// vyber hrace
	CGUIPromptList<CSokobanSceneMenu>* promptListSelectPlayer =
			new CGUIPromptList<CSokobanSceneMenu> ();
	promptListSelectPlayer->initChilds();
	promptListSelectPlayer->setDimension(200, 150);
	promptListSelectPlayer->centered();
	promptListSelectPlayer->setCallback(this, &CSokobanSceneMenu::selectPlayer);
	promptListSelectPlayer->hide();

	m_gameObject2DManager->add(promptListSelectPlayer);
	m_referencesObject2D[10] = m_gameObject2DManager->getLastInsertID();

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Vstupy a výběry

	CGUILabel* playerName = new CGUILabel();
	CGUILabel* playerLevel = new CGUILabel();
	CGUILabel* playerScore = new CGUILabel();

	playerName->setPosition(100, 500);
	playerLevel->setPosition(300, 500);
	playerScore->setPosition(500, 500);

	m_gameObject2DManager->add(playerName);
	m_referencesObject2D[7] = m_gameObject2DManager->getLastInsertID();
	m_gameObject2DManager->add(playerLevel);
	m_referencesObject2D[8] = m_gameObject2DManager->getLastInsertID();
	m_gameObject2DManager->add(playerScore);
	m_referencesObject2D[9] = m_gameObject2DManager->getLastInsertID();

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------


	// -------------------------------------------------------------------------
	// Načtení textur

	int logoTextureId = CTextureManager::getInstance()->createTexture(
			"data/textures/ground.jpg", false);
	int wallTexture = CTextureManager::getInstance()->createTexture(
			"data/textures/wall2.png", false);

	int crateTexture = CTextureManager::getInstance()->createTexture(
			"data/textures/crate.jpg", false);

	int boxTexture = CTextureManager::getInstance()->createTexture(
			"data/textures/box.jpg", false);

	int cursorTex = CTextureManager::getInstance()->createTexture(
			"data/textures/normal.png", false);

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------


	// -------------------------------------------------------------------------
	// Nastavení kurzoru
	CCursor* cursor = new CCursor();
	cursor->setDimension(32, 32);
	cursor->setTextureId(cursorTex - 1);

	m_gameObject2DManager->add(cursor);
	m_referencesObject2D[6] = m_gameObject2DManager->getLastInsertID();

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------


	// -------------------------------------------------------------------------
	// Level v menu

	CLevel* level = new CLevel();
	level->setTextureId(-5);
	level->setCrateTexId(crateTexture - 1);
	level->setBoxTexId(boxTexture - 1);

	m_level = level;

	// nacteni levelu ze souboru
	CLevelLoader* ll = CLevelLoader::getInstance();

	if (ll->loadLevel("data/levels/1")) {
		ll->put(*(this->m_level));
	}

	level->setWallTexId(0, logoTextureId - 1);
	level->setWallTexId(1, wallTexture - 1);
	level->setWallTexId(2, wallTexture - 1);
	level->setWallTexId(3, wallTexture - 1);
	level->setWallTexId(4, wallTexture - 1);

	this->m_gameObject3DManager->add(level);

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------


	// -------------------------------------------------------------------------
	// Nastavení kamery v menu

	CCamera *camera = CCamera::getInstance();
	camera -> setPosition(10, 8, -51);
	camera -> setTarget(0, 0, 0);
	camera ->setAngle(180);

	GLfloat angle = camera->getAngle();

	camera->setPositionZ(-cos(angle * PI / 180) * 5);
	camera->setPositionX(sin(angle * PI / 180) * 5);

	CRenderer::getInstance()->blendingEnable();

	// -------------------------------------------------------------------------
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// ---- Nacteni hracskeho profilu

	this->hidePlayerInfo();

	CProfileManager *profiles = CProfileManager::getInstance();

	if (profiles->isSetProfile()) {
		showMenu();
		this->setPlayerInfo(profiles->getActiveProfileName(), profiles->getActiveProfileLevel(), profiles->getActiveProfileScore());
	} else {

		if (profiles->count() > 0) {
			promptListSelectPlayer->centered();
			requestSelectPlayer();
		} else {
			requestCreateNewPlayer();
		}
	}

	prepareProfiles();

	// ----
	this->initialized = true;
}

void CSokobanSceneMenu::clear() {
	this->m_gameObject2DManager->clean();
	this->m_gameObject3DManager->clean();

	CTextureManager::getInstance()->clean();
}

void CSokobanSceneMenu::updateScene() {

	CCamera *camera = CCamera::getInstance();
	GLfloat angle = camera->getAngle();

	camera->setPositionZ(-cos(angle) * 5);
	camera->setPositionX(sin(angle) * 5);
	camera->setAngle(angle + 0.005f);

	if (m_fadeIn) {
		m_fadesAnimation.update();

		if (!m_fadesAnimation.isEnd()) {
			CGameObject2D *background = this->getReferencedObject2D(0);
			if (background != NULL) {
				background->setAlpha(
						255 - m_fadesAnimation.getCurrentFrame() / 2);
			}
		} else {
			showHeader();
			showPlayerInfo();
			m_fadeIn = false;
		}
	}

	if (m_fadeOut) {
		m_fadesAnimation.update();

		if (!m_fadesAnimation.isEnd()) {
			CGameObject2D *background = this->getReferencedObject2D(0);
			if (background != NULL) {
				cout << background->getAlpha();
				background->setAlpha(
						background->getAlpha() * 255
								+ m_fadesAnimation.getCurrentFrame() / 4);
			}
		} else {
			m_fadeOut = false;
			this->initialized = false;
			this->m_running = false;
		}
	}

}

void CSokobanSceneMenu::keyboardSpecial(int key, int x, int y) {

}

// --

void CSokobanSceneMenu::playGame() {

	hidePlayerInfo();
	hideHeader();
	hideMenu();

	this->m_fadesAnimation.reset();
	this->m_fadesAnimation.play();
	this->m_fadeOut = true;
}

void CSokobanSceneMenu::quitGame(bool result) {

	if (result) {
		SokobanGame::getInstance()->terminate();
	}

	showMenu();
	hideConfQuitGame();
}

void CSokobanSceneMenu::resetGame(bool result) {

	if (result) {
		CProfileManager* profiles = CProfileManager::getInstance();
		if (profiles != NULL ) {
			profiles->resetPlayerGame();
			this->setPlayerInfo(profiles->getActiveProfileName(),
							profiles->getActiveProfileLevel(),
							profiles->getActiveProfileScore());
		}
	}

	showMenu();
	hideConfResetGame();

}

void CSokobanSceneMenu::selectPlayer(bool result) {

	CProfileManager* profiles = CProfileManager::getInstance();

	if (result) {
		CGameObject2D* promp = this->getReferencedObject2D(10);
		int
				playerID =
						dynamic_cast<CGUIPromptList<CSokobanSceneMenu>*> (promp)->getValue();

		if (playerID == -1) {
			hidePromptSelectPlayer();
			requestCreateNewPlayer();
		} else {

			if (profiles->setActiveProfile(playerID)) {
				showMenu();
				hidePromptSelectPlayer();
				this->setPlayerInfo(profiles->getActiveProfileName(),
						profiles->getActiveProfileLevel(),
						profiles->getActiveProfileScore());
			} else {

			}
		}

	} else {
		if (profiles->isSetProfile()) {
			showMenu();
			hidePromptSelectPlayer();
		}
	}
}

void CSokobanSceneMenu::createNewPlayer(bool result) {

	CGameObject2D* promp = this->getReferencedObject2D(5);
	CProfileManager* profiles = CProfileManager::getInstance();

	if (result) {

		string
				strName =
						dynamic_cast<CGUIPrompt<CSokobanSceneMenu>*> (promp)->getResult(); // vezmeme retezec z vstupu

		if (strName == "") { // v pripade, ze je prazdny
			cout << "WE WAIT" << endl;
			return; // pockame, dokud tam neco uzivatel nenapise
		}

		int newPlayerID = profiles->createNewPlayer(strName); // vytvorime novy uzivatelsky profil
		profiles->setActiveProfile(newPlayerID); // nastavime jej jako aktivni

		CGameObject2D* list = this->getReferencedObject2D(10);
		dynamic_cast<CGUIPromptList<CSokobanSceneMenu>*> (list)->addOption(newPlayerID, strName);

		this->setPlayerInfo(profiles->getActiveProfileName(),
				profiles->getActiveProfileLevel(),
				profiles->getActiveProfileScore());
		this->showPlayerInfo();

		// TODO nastaveni labelu: jmeno hrace, level, score

		hidePromptCreateNewPlayer();
		showMenu();

	} else {

		if (profiles->count() == 0) { // v pripade, ze neni vytvoreny zadny uzivatelsky profil
			return; // pockame dokud nebude vytvoren
		} else {

			if (profiles->isSetProfile()) {
				showMenu();
			} else {
				showPromptSelectPlayer();
			}

			hidePromptCreateNewPlayer();
		}

	}

}

// requests

void CSokobanSceneMenu::requestQuitGame() {
	showConfQuitGame();
	hideMenu();
}

void CSokobanSceneMenu::requestResetGame() {
	showConfResetGame();
	hideMenu();
}

void CSokobanSceneMenu::requestCreateNewPlayer() {
	showPromptCreateNewPlayer();
	hideMenu();
}

void CSokobanSceneMenu::requestSelectPlayer() {
	showPromptSelectPlayer();
	hideMenu();
}

void CSokobanSceneMenu::showPlayerInfo() {
	showSceneObject2D(7);
	showSceneObject2D(8);
	showSceneObject2D(9);
}

void CSokobanSceneMenu::showHeader() {
	showSceneObject2D(1);
}

void CSokobanSceneMenu::showMenu() {
	showSceneObject2D(2);
}

void CSokobanSceneMenu::showConfQuitGame() {
	showSceneObject2D(4);
}

void CSokobanSceneMenu::showConfResetGame() {
	showSceneObject2D(3);
}

void CSokobanSceneMenu::showPromptCreateNewPlayer() {
	showSceneObject2D(5);
}

void CSokobanSceneMenu::showPromptSelectPlayer() {
	showSceneObject2D(10);
}

void CSokobanSceneMenu::hidePlayerInfo() {
	hideSceneObject2D(7);
	hideSceneObject2D(8);
	hideSceneObject2D(9);
}

void CSokobanSceneMenu::hideMenu() {
	hideSceneObject2D(2);
}

void CSokobanSceneMenu::hideConfQuitGame() {
	hideSceneObject2D(4);
}

void CSokobanSceneMenu::hideConfResetGame() {
	hideSceneObject2D(3);
}

void CSokobanSceneMenu::hidePromptCreateNewPlayer() {
	hideSceneObject2D(5);
}

void CSokobanSceneMenu::hidePromptSelectPlayer() {
	hideSceneObject2D(10);
}

void CSokobanSceneMenu::hideHeader() {
	hideSceneObject2D(1);
}

void CSokobanSceneMenu::setPlayerInfo(string name, int level, int score) {

	CGameObject2D* labelName = this->getReferencedObject2D(7);
	CGameObject2D* labelLevel = this->getReferencedObject2D(8);
	CGameObject2D* labelScore = this->getReferencedObject2D(9);

	if (labelName != NULL) {
		cout << "SETUP PLAYER NAME TO: " << name << endl;
		string stringName("PLAYER: ");
		stringName.append(name);
		dynamic_cast<CGUILabel*> (labelName)->setString(stringName);
	}

	if (labelLevel != NULL) {
		cout << "SETUP PLAYER NAME TO: " << level << endl;
		string stringLevel("LEVEL: ");
		stringLevel.append(_toString<int> (level));
		dynamic_cast<CGUILabel*> (labelLevel)->setString(stringLevel);
	}

	if (labelScore != NULL) {
		cout << "SETUP PLAYER NAME TO: " << score << endl;
		string stringScore("SCORE: ");
		stringScore.append(_toString<int> (score));
		dynamic_cast<CGUILabel*> (labelScore)->setString(stringScore);
	}

}

void CSokobanSceneMenu::prepareProfiles() {

	CProfileManager* profiles = CProfileManager::getInstance();
	CProfileManager::SPlayerInfo* currProfile = NULL;
	CGameObject2D* list = this->getReferencedObject2D(10);

	if (list != NULL) {
		for (int i = 0; i < profiles->count(); i++) {
			currProfile = profiles->getPlayerProfile(i);
			if (currProfile != NULL) {
				dynamic_cast<CGUIPromptList<CSokobanSceneMenu>*> (list)->addOption(currProfile->m_id, currProfile->m_name);
			}
		}
	}

}
