/** @file SokobanSceneMenu.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CSokobanSceneMenu
 *
 *
 * \brief Scéna - menu
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CSOKOBANSCENEMENU_H_
#define _CSOKOBANSCENEMENU_H_

#include "Level.h"
#include "GameState.h"
#include "GUIForm.h"

#include "Animation.h"

class CSokobanSceneMenu: public CGameState {
public:
	CSokobanSceneMenu();
	virtual ~CSokobanSceneMenu();

	void init();
	void clear();
	void updateScene();

	void keyboard(unsigned char key, int x, int y) {
		CGameState::keyboard(key, x, y);
	}

	void keyboardSpecial(int key, int x, int y);

	// --
	void playGame();
	void quitGame(bool result);
	void resetGame(bool result);
	void createNewPlayer(bool result);
	void selectPlayer(bool result);

	void requestQuitGame();
	void requestResetGame();
	void requestCreateNewPlayer();
	void requestSelectPlayer();

	void showPlayerInfo();
	void showHeader();
	void showMenu();
	void showConfQuitGame();
	void showConfResetGame();
	void showPromptCreateNewPlayer();
	void showPromptSelectPlayer();

	void hidePlayerInfo();
	void hideHeader();
	void hideMenu();
	void hideConfQuitGame();
	void hideConfResetGame();
	void hidePromptCreateNewPlayer();
	void hidePromptSelectPlayer();

	void prepareProfiles();
	void setPlayerInfo(string name, int level, int score);

protected:
	CLevel* m_level;

	/**
	 * Fade in
	 */
	bool m_fadeIn;

	/**
	 * Fade out
	 */
	bool m_fadeOut;

	/**
	 * Animace
	 */
	CAnimation m_fadesAnimation;

};

#endif /* _CSOKOBANSCENEMENU_H_ */
