/** @file Textureable.h
 *
 * \author František Toman
 *
 * \version 1.0
 *
 */

/**
 * \class CTextureable
 *
 *
 * \brief Otexturování
 *
 *
 * \author František Toman
 *
 * \version 1.0
 *
 *
 * Kontakt: tomanfra@fit.cvut.cz
 *
 */

#ifndef _CTEXTUREABLE_H_
#define _CTEXTUREABLE_H_

#include "main.h"

class CTextureable {
public:
	CTextureable();
	~CTextureable();

	/**
	 * Nastaví ID textury
	 * @param textureId
	 */
	void setTextureId(int textureId) {
		m_textureId = textureId;
	}
	;

	/**
	 * Vrátí ID textury - odkaz na texturu ve správci textur (TextureManager)
	 * @return
	 */
	int getTextureId() const {
		return m_textureId;
	}
	;

	/**
	 * Je objekt otexturován?
	 * @return
	 */
	bool isTextured() const {
		return m_textureId >= 0 ? true : false;
	}
	; // vraci, zdali je herni objekt otexturovan, resp. je mu prirazena textura

	/**
	 * Jsou nastaveny souřadnice textury
	 * @return
	 */
	bool isSetVertices() const {
		return m_texCoord == NULL ? false : true;
	}
	;

	/**
	 * Vrátí ukazatel na pole souřadnic textury
	 * @return
	 */
	GLfloat *getVertices() const {
		return m_texCoord;
	}
	;

	/**
	 * Nastaví pole souřadnic texturz
	 * @param vertices
	 * @param count
	 */
	void setVertices(GLfloat* vertices, int count);

	/**
	 * Dealokace prostředků
	 */
	void clean();

	/**
	 * Inicializace prostředků
	 * @param count
	 */
	void init(int count);

protected:
	int m_textureId;
	GLfloat* m_texCoord; // souradnice textury
	int m_countVertices; // pocet vrcholu

};

#endif /* _CTEXTUREABLE_H_ */
